@@ -1,5 +1,3 @@
-/* -*- mode:C; indent-tabs-mode:nil; tab-width:8; c-basic-offset:8 -*- */
-
 /*
  * Gyatzee: Gnomified Dice game.
  * (C) 1998 the Free Software Foundation
@@ -44,11 +42,10 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include <libgnome-games-support.h>
 
 #include "yahtzee.h"
 #include "gyahtzee.h"
-#include "games-scores.h"
-#include "games-scores-dialog.h"
 
 #define DELAY_MS 600
 
@@ -113,14 +110,40 @@ static const GOptionEntry yahtzee_options[] = {
   {NULL}
 };
 
-static const GamesScoresCategory category_array[] = {
-  {"Regular", NC_("game type", "Regular") },
+typedef struct
+{
+  gchar *key;
+  gchar *name;
+} key_value;
+
+static const key_value category_array[] = {
+  /* Order must match GameType enum order */
+  {"Regular", NC_("game type", "Regular")},
   {"Colors",  NC_("game type", "Colors")}
 };
 
-GamesScores *highscores;
+const gchar *
+category_name_from_key (const gchar *key)
+{
+  int i;
+  for (i = 0; i < G_N_ELEMENTS (category_array); ++i) {
+    if (g_strcmp0 (category_array[i].key, key) == 0)
+      return g_dpgettext2(NULL, "game type", category_array[i].name);
+  }
+  return NULL;
+}
 
-static GtkWidget *dialog = NULL;
+static GamesScoresCategory *
+create_category_from_key (const gchar *key,
+                          gpointer     user_data)
+{
+  const gchar *name = category_name_from_key (key);
+  if (name == NULL)
+    return NULL;
+  return games_scores_category_new (key, name);
+}
+
+GamesScoresContext *highscores;
 
 static void modify_dice (GtkToolButton * widget, gpointer data);
 static void UpdateRollLabel (void);
@@ -153,13 +176,26 @@ update_roll_button_sensitivity (void)
   gtk_widget_set_sensitive (GTK_WIDGET (mbutton), state);
 }
 
+static void
+add_score_cb (GObject      *source_object,
+              GAsyncResult *res,
+              gpointer      user_data)
+{
+  GamesScoresContext *context = GAMES_SCORES_CONTEXT (source_object);
+  GError *error = NULL;
+
+  games_scores_context_add_score_finish (context, res, &error);
+  if (error != NULL) {
+    g_warning ("Failed to add score: %s", error->message);
+    g_error_free (error);
+  }
+}
+
 static void
 CheerWinner (void)
 {
   int winner;
   int i;
-  gint pos;
-  gchar *message;
 
   ShowoffPlayer (ScoreList, CurrentPlayer, 0);
 
@@ -180,25 +216,11 @@ CheerWinner (void)
   ShowoffPlayer (ScoreList, winner, 1);
 
   if (winner < NumberOfHumans) {
-    pos = games_scores_add_plain_score (highscores, (guint32) WinningScore);
-
-    if (pos > 0) {
-      if (dialog) {
-        gtk_window_present (GTK_WINDOW (dialog));
-      } else {
-        dialog = games_scores_dialog_new (GTK_WINDOW (window), highscores, _("Tali Scores"));
-        message =
-  	  g_strdup_printf ("<b>%s</b>\n\n%s", _("Congratulations!"),
-                           pos == 1 ? _("Your score is the best!") :
-                           _("Your score has made the top ten."));
-        games_scores_dialog_set_message (GAMES_SCORES_DIALOG (dialog), message);
-        g_free (message);
-      }
-      games_scores_dialog_set_hilight (GAMES_SCORES_DIALOG (dialog), pos);
-
-      gtk_dialog_run (GTK_DIALOG (dialog));
-      gtk_widget_hide (dialog);
-    }
+    GamesScoresCategory *category;
+    category = create_category_from_key (category_array[game_type].key, NULL);
+    games_scores_context_add_score (highscores, (guint32) WinningScore,
+                                    category, NULL,
+                                    add_score_cb, NULL);
   }
 
   if (players[winner].name)
@@ -207,8 +229,8 @@ CheerWinner (void)
 	 players[winner].name, WinningScore);
   else
     say (_("Game over!"));
-
 }
+
 static gboolean
 do_computer_turns (void)
 {
@@ -448,7 +470,6 @@ GyahtzeeNewGame (void)
   say (_("Select dice to roll or choose a score slot."));
 
   game_type = get_new_game_type ();
-  games_scores_set_category (highscores, SCORES_CATEGORY);
   NewGame ();
   setup_score_list (ScoreList);
   UpdateRollLabel ();
@@ -625,11 +646,7 @@ about_cb (GSimpleAction * action, GVariant * parameter, gpointer data)
 void
 ShowHighScores (void)
 {
-  if (!dialog)
-    dialog = games_scores_dialog_new (GTK_WINDOW (window), highscores, _("Tali Scores"));
-
-  gtk_dialog_run (GTK_DIALOG (dialog));    
-  gtk_widget_hide (dialog);
+  games_scores_context_run_dialog (highscores);
 }
 
 static void
@@ -698,7 +715,7 @@ help_cb (GSimpleAction * action, GVariant * parameter, gpointer data)
 {
   GError *error = NULL;
 
-  gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (window)), "help:tali", gtk_get_current_event_time (), &error);
+  gtk_show_uri_on_window (GTK_WINDOW (window), "help:tali", gtk_get_current_event_time (), &error);
   if (error)
     g_warning ("Failed to show help: %s", error->message);
   g_clear_error (&error);
@@ -855,6 +872,17 @@ GyahtzeeCreateMainWindow (GApplication *app, gpointer user_data)
 static void
 GyahtzeeActivateGame (GApplication *app, gpointer user_data)
 {
+  GamesScoresImporter *importer;
+
+  importer = GAMES_SCORES_IMPORTER (games_scores_directory_importer_new ());
+  highscores = games_scores_context_new_with_importer ("tali",
+                                                       _("Game Type:"),
+                                                       GTK_WINDOW (window),
+                                                       create_category_from_key, NULL,
+                                                       GAMES_SCORES_STYLE_POINTS_GREATER_IS_BETTER,
+                                                       importer);
+  g_object_unref (importer);
+
   if (!gtk_widget_is_visible (window)) {
     gtk_widget_show (window);
     GyahtzeeNewGame ();
@@ -880,8 +908,6 @@ main (int argc, char *argv[])
   g_signal_connect (application, "startup", G_CALLBACK (GyahtzeeCreateMainWindow), NULL);
   g_signal_connect (application, "activate", G_CALLBACK (GyahtzeeActivateGame), NULL);
 
-  games_scores_startup ();
-
   /* Reset all yahtzee variables before parsing args */
   YahtzeeInit ();
 
@@ -940,12 +966,6 @@ main (int argc, char *argv[])
       exit (0);
   }
 
-  highscores = games_scores_new ("tali",
-                                 category_array, G_N_ELEMENTS (category_array),
-                                 "game type", NULL,
-                                 0 /* default category */,
-                                 GAMES_SCORES_STYLE_PLAIN_DESCENDING);
-
   gtk_window_set_default_icon_name ("org.gnome.Tali");
 
   if (NumberOfComputers == 0)	/* Not set on the command-line. */
@@ -1012,11 +1032,3 @@ main (int argc, char *argv[])
 
   exit (0);
 }
-
-/* Arrgh - lets all use the same tabs under emacs: 
-Local Variables:
-tab-width: 8
-c-basic-offset: 8
-indent-tabs-mode: nil
-End:
-*/
